package jess;

import java.util.*;

/**
 * The HashCodeComputer returns hash code values for objects depending
 * on their status as instances of "value" or "non-value" classes.
 * Assume everything is a nonvalue class by default? Check for equals/hashCode overrides?

 * <P>
 * (C) 2006 Sandia National Laboratories<br>
 */

class HashCodeComputer {

    private static List m_nonValueClasses = Collections.synchronizedList(new ArrayList());
    static {
        m_nonValueClasses.add(Map.class);
        m_nonValueClasses.add(Collection.class);
    }

    static int hashCode(Object o) {
        if (isValueObject(o))
            return o.hashCode();
        else
            return 0;
    }

    // TODO Use a special kind of Value wrapper for mutable-hashcode classes, applied at definstance time?
    static boolean isValueObject(Object o) {
        synchronized(m_nonValueClasses) {
            for (int i=0; i< m_nonValueClasses.size(); ++i) {
                Class clazz  = (Class) m_nonValueClasses.get(i);
                if (clazz.isAssignableFrom(o.getClass()))
                    return false;
            }
            return true;
        }
    }

    public static void setNonValueClass(Rete engine, String name) throws ClassNotFoundException {
        final Class aClass = engine.findClass(name);
        synchronized(m_nonValueClasses) {
            if (!m_nonValueClasses.contains(aClass))
                m_nonValueClasses.add(aClass);
        }
    }
}
