package jess;

import java.io.Serializable;
import java.util.TreeMap;
import java.util.Iterator;

/**
 * The actual agenda for a single module
 * <P>
 * (C) 2006 Sandia National Laboratories<BR>
 */


class ModuleAgenda implements Serializable {
    private final HeapPriorityQueue m_queue;
    private String m_name;


    ModuleAgenda(String name, Strategy s) {
        m_name = name;
        m_queue = new HeapPriorityQueue(s);
    }

    String setStrategy(Strategy s, Rete engine) throws JessException {
        synchronized (m_queue) {
            Iterator e = m_queue.iterator(engine);
            m_queue.clear();
            String name = m_queue.setStrategy(s).getName();
            while (e.hasNext()) {
                Activation a = (Activation) e.next();
                if (!a.isInactive())
                    engine.addActivation(a);
            }
            return name;
        }
    }

    HeapPriorityQueue getQueue() {
        return m_queue;
    }

    /**
     * Returns the name of this module
     * @return the name
     */
    public String getName() {
        return m_name;
    }

    void reset() {
        m_queue.clear();
    }

    void confirmStrategy(Rete engine, Strategy strategy) throws JessException {
        if (m_queue.getStrategy() != strategy) {
            setStrategy(strategy, engine);
        }
    }
}
