package jess;

import java.beans.IntrospectionException;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

/**
 * (C) 2006 Sandia National Laboratories
 */
public class ReflectionClassResearcher implements ClassResearcher {
    private Rete m_engine;

    public ReflectionClassResearcher(Rete engine) {
        m_engine = engine;
    }

    public String resolveClassName(String clazz) throws ClassNotFoundException {
        Class theClass = m_engine.findClass(clazz);
        return theClass.getName();
    }

    public Property[] getBeanProperties(String clazz) throws ClassNotFoundException, JessException {
        Class c = m_engine.findClass(clazz);
        try {
            PropertyDescriptor[] props = ReflectFunctions.getPropertyDescriptors(c);
            ArrayList properties = new ArrayList();
            for (int i = 0; i < props.length; i++) {
                PropertyDescriptor pd = props[i];
                Method m = props[i].getReadMethod();
                if (m == null)
                    continue;
                String name = pd.getName();
                Class type = pd.getPropertyType();
                boolean isArray = type.isArray();
                if (isArray)
                    type = type.getComponentType();
                String jessType = getSlotType(type);

                properties.add(new Property(name, jessType, isArray));
            }
            return (Property[]) properties.toArray(new Property[properties.size()]);

        } catch (IntrospectionException e) {
            throw new JessException("ReflectionClassResearcher.getBeanProperties", "Introspector threw an exception", e);
        }

    }

    public Property[] getPublicInstanceFields(String clazz) throws ClassNotFoundException, JessException {
        Class c = m_engine.findClass(clazz);
        Field[] fields = c.getFields();
        ArrayList properties = new ArrayList();
        for (int i = 0; i < fields.length; i++) {
            Field f = fields[i];
            if (Modifier.isStatic(f.getModifiers()))
                continue;

            String name = f.getName();
            Class type = f.getType();
            boolean isArray = type.isArray();
            if (isArray)
                type = type.getComponentType();
            String jessType = getSlotType(type);

            properties.add(new Property(name, jessType, isArray));
        }
        return (Property[]) properties.toArray(new Property[properties.size()]);
    }

    static String getSlotType(Class type) {
        if (type == int.class || type == Integer.class ||
                type == short.class || type == Short.class ||
                type == byte.class || type == Byte.class)
            return "INTEGER";
        else if (type == long.class || type == Long.class)
            return "LONG";
        else if (type == boolean.class || type == Boolean.class)
            return "SYMBOL";
        else if (type == float.class || type == Float.class ||
                type == double.class || type == Double.class)
            return "FLOAT";
        else if (type == String.class)
            return "STRING";
        else
            return "ANY";
    }
}
