package jess;

import java.util.HashMap;
import java.io.Serializable;

/**
 * (C) 2006 Sandia National Laboratories
 */
class ValueFactory implements Serializable {

    private final HashMap s_symbols = new HashMap();

    public ValueFactory() {
        Value[] constants = {
                Funcall.TRUE, Funcall.FALSE, Funcall.NIL,
                Funcall.EOF, Funcall.T, Funcall.CRLF,
                Funcall.s_else, Funcall.s_elif,
                Funcall.s_then, Funcall.s_do
        };
        try {
            for (int i = 0; i < constants.length; i++) {
                Value constant = constants[i];
                get(constant.symbolValue(null), constant.type());
            }
        } catch (JessException cantHappen) {
            // Silently ignore
        }

    }

    public Value get(String v, int type) throws JessException {
        switch (type) {
            case RU.VARIABLE:
            case RU.MULTIVARIABLE:
                return new Variable(v, type);

            case RU.SYMBOL: {
                synchronized(s_symbols) {
                    Value symbol = (Value) s_symbols.get(v);
                    if (symbol == null) {
                        symbol = new Value(v, RU.SYMBOL);
                        s_symbols.put(v, symbol);
                    }
                    return symbol;
                }
            }

            default:
                return new Value(v, type);

        }
    }

    public Value get(double v, int type) throws JessException {
        return new Value(v, type);
    }

    public Value get(int v, int type) throws JessException {
        return new Value(v, type);
    }

    public Value get(int v) throws JessException {
        return new Value(v);
    }

    public Value get(long v) throws JessException {
        return new LongValue(v);
    }

    public Value get(boolean v) throws JessException {
        return v ? Funcall.TRUE : Funcall.FALSE;
    }

    public Value get(Object v) throws JessException {
        return v == null ? Funcall.NIL : new Value(v);
    }
}
