package jess.speedup;

import jess.*;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Opcodes;

import java.io.Serializable;
import java.util.HashMap;

/**
 * A simple implementation of Accelerator which speeds up integer math using the ASM bytecode library
 * <P>
 * (C) 2006 Sandia National Laboratories<br>
 */

public class ASMAccelerator extends ClassLoader implements Accelerator, Serializable, Opcodes {
    private static int s_index;

    public TestBase speedup(Funcall f, Rete engine) throws JessException {
        if (f.getName().equals("eq")
                &&
                f.size() == 3 &&
                f.get(1).type() == RU.BINDING &&
                f.get(2).type() == RU.BINDING)
            return generateTest(f);
        else
            return null;
    }

    private TestBase generateTest(Funcall f) throws JessException {
        ClassWriter cw = new ClassWriter(ClassWriter.COMPUTE_FRAMES | ClassWriter.COMPUTE_MAXS);
        MethodVisitor mv;
        String className = makeClassName();
        System.out.println(className);
        cw.visit(V1_4, ACC_PUBLIC + ACC_SUPER, "jess/tmp/" + className, null, "java/lang/Object", new String[]{"jess/TestBase"});

        HashMap vars = new HashMap();
        int max = fetchVariables(f, vars, 0);
        mv = cw.visitMethod(ACC_PUBLIC, "doTest", "(Ljess/Context;)Z", null, new String[]{"jess/JessException"});
        mv.visitCode();

        // Token lt = context.getToken();
        mv.visitVarInsn(ALOAD, 1);
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/Context", "getToken", "()Ljess/Token;");
        mv.visitVarInsn(ASTORE, 2);

        // ValueVector rf = context.getFact();
        mv.visitVarInsn(ALOAD, 1);
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/Context", "getFact", "()Ljess/Fact;");
        mv.visitVarInsn(ASTORE, 3);

        BindingValue op1 = (BindingValue) f.get(1);
        BindingValue op2 = (BindingValue) f.get(2);

        // op1 should be token argument, op2 from fact
        if (op1.getFactNumber() == max) {
            BindingValue tmp = op2;
            op2 = op1;
            op1 = tmp;
        }

        // Arg from right fact into R4
        mv.visitVarInsn(ALOAD, 3);
        mv.visitLdcInsn(new Integer(op2.getSlotIndex()));
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/ValueVector", "get", "(I)Ljess/Value;");
        mv.visitVarInsn(ASTORE, 4);

        // Arg from token into R5
        mv.visitVarInsn(ALOAD, 2);
        mv.visitLdcInsn(new Integer(op1.getFactNumber()));
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/Token", "fact", "(I)Ljess/Fact;");
        mv.visitLdcInsn(new Integer(op1.getSlotIndex()));                        
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/Fact", "get", "(I)Ljess/Value;");
        mv.visitVarInsn(ASTORE, 5);

        mv.visitVarInsn(ALOAD, 5);
        mv.visitVarInsn(ALOAD, 4);
        mv.visitMethodInsn(INVOKEVIRTUAL, "jess/Value", "equals", "(Ljess/Value;)Z");
        mv.visitInsn(IRETURN);
        mv.visitMaxs(0, 0);
        mv.visitEnd();

        {
            mv = cw.visitMethod(ACC_PUBLIC, "<init>", "()V", null, null);
            mv.visitCode();
            mv.visitVarInsn(ALOAD, 0);
            mv.visitMethodInsn(INVOKESPECIAL, "java/lang/Object", "<init>", "()V");
            mv.visitInsn(RETURN);
            mv.visitMaxs(1, 1);
            mv.visitEnd();
        }
        cw.visitEnd();

        cw.visitSource("jess/tmp/" + className + ".java", null);
        byte[] b = cw.toByteArray();

        try {
            // FileOutputStream fos = new FileOutputStream("jess/tmp/" + className + ".class");
            // fos.write(b);
            // fos.close();
            Class clazz = defineClass("jess.tmp." + className, b, 0, b.length);
            return (TestBase) clazz.newInstance();
        } catch (Exception e) {
            throw new JessException("ASMAccelerator::generateTest", "Error loading test class", e);
        }
    }

    static String makeClassName() {
        return "ASMTest" + s_index++;
    }

  private static int fetchVariables(Funcall f, HashMap vars, int max)
    throws JessException
  {
    for (int i=1; i<f.size(); i++)
      {
        Value v = f.get(i);
        switch (v.type())
          {
          case RU.BINDING:
            {
              BindingValue bv = (BindingValue) v;
              Integer hash = new Integer(v.hashCode());
              if (vars.get(hash) == null)
                {
                  vars.put(hash, v);
                  if (bv.getFactNumber() > max)
                    max = bv.getFactNumber();
                }

              break;
            }
          case RU.FUNCALL:
            {
              max = fetchVariables(v.funcallValue(null), vars, max);
              break;
            }
          }
      }
    return max;
  }


}
